import zipfile
import io

import requests

def _get_zip(url):
    """Retrieve zipfile using http request.

    zipfiles

    Parameters
    ----------
    url : type
        Description of parameter `url`.

    Returns
    -------
    zipfile.ZipFile

    Usage:

    >>> zip = _get_zip('https://www.bls.gov/oes/special.requests/oesm17nat.zip')
    >>> type(zip)
    <class 'zipfile.ZipFile'>

    >>> for file in zip.filelist:
    ...     print(file.filename)
    ...
    oesm17nat/field_descriptions.xlsx
    oesm17nat/national_M2017_dl.xlsx

    # accessing zipfile obects for i/o operations

    >>> fp = zip.open('oesm17nat/field_descriptions.xlsx', 'r')
    >>> fp.readable()
    True

    fp.close()

    """
    try:
        r = requests.get(url)
        if r.ok:
            return zipfile.ZipFile(io.BytesIO(r.content))
        else:
            raise requests.exceptions.HTTPError('Not a valid request.'
                'Double check for valid parameters.')
    except requests.exceptions.ConnectionError as e:
        print(e, '\nFailed to establish a new connection. Check internet.')

def get_file(url, filename):
    try:
        zip = _get_zip(url)
        fp = zip.open(filename, 'r')
        return fp
    except Exception as e:
        print(e)



import io

import requests
import pandas as pd

def _get_area_url(**kwargs):
    '''Get url for QCEW data by area.

    Parameters
    ----------
    year : string
        'YYYY'
    qtr : string
        Use `a` for annual averages.
    area : string
        For all area codes and titles see:
        http://www.bls.gov/cew/doc/titles/area/area_titles.htm.

    Returns
    -------
    string
        URL that will be used for main function.

    '''

    for arg in ['year', 'qtr', 'area']:
        if arg not in kwargs:
            raise ValueError('Must specify {}'.format(arg))

    url = 'http://data.bls.gov/cew/data/api/[YEAR]/[QTR]/area/[AREA].csv'
    url = url.replace('[YEAR]', kwargs.get('year'))
    url = url.replace('[QTR]', kwargs.get('qtr').lower())
    url = url.replace('[AREA]', kwargs.get('area').upper())
    return url

def _get_industry_url(**kwargs):
    '''Get url for QCEW data by industry.

    Parameters
    ----------
    year : string
        'YYYY'
    qtr : string
        Use `a` for annual averages.
    industry : string
        For all industry codes and titles see:
        http://www.bls.gov/cew/doc/titles/industry/industry_titles.htm

    Returns
    -------
    string
        URL that will be used for main function.

    '''

    for arg in ['year', 'qtr', 'industry']:
        if arg not in kwargs:
            raise ValueError('Must specify {}'.format(arg))

    url = 'http://data.bls.gov/cew/data/api/[YEAR]/[QTR]/industry/[IND].csv'
    url = url.replace('[YEAR]', kwargs.get('year'))
    url = url.replace('[QTR]', kwargs.get('qtr').lower())
    url = url.replace('[IND]', kwargs.get('industry'))
    return url


def _get_size_url(**kwargs):
    '''Get url for QCEW data by size.

    Note that size data is only available for the first quarter of each year
    Parameters
    ----------
    year : string
        'YYY'.
    size : string
        For all establishment size classes and titles see:
        http://www.bls.gov/cew/doc/titles/size/size_titles.htm

    Returns
    -------
    string
        URL that will be used for main function.

    '''

    for arg in ['year', 'size']:
        if arg not in kwargs:
            raise ValueError('Must specify {}'.format(arg))

    url = 'http://data.bls.gov/cew/data/api/[YEAR]/1/size/[SIZE].csv'
    url = url.replace('[YEAR]', kwargs.get('year'))
    url = url.replace('[SIZE]', kwargs.get('size'))
    return url

def get_data(cut_by, rtype='dataframe', **kwargs):
    """Get a dataset from the Quarterly Census of Employment and Wages (QCEW)

    Parameters
    ----------
    cut_by : str
        Specify QCEW data cut/sliced. Occupations can be cut/sliced by industry,
        area, and size class
    rtype : str
        Default behavior is to return a pandas dataframe.
        If you want a primitive i/o csv file, then specify rtype='csv'.
    **kwargs :
        Allows `get_data()` to act as a wrapper funciton.
        Arguments vary by how the data is being cut/sliced

        Required argument by data cut/slice

        Area: year, qtr, area
            For all area codes and titles see:
            http://www.bls.gov/cew/doc/titles/area/area_titles.htm.
        Industry: year, qtr, industry
            For all industry codes and titles see:
            http://www.bls.gov/cew/doc/titles/industry/industry_titles.htm
        Size: year, size
            For all establishment size classes and titles see:
            http://www.bls.gov/cew/doc/titles/size/size_titles.htm


    Returns
    -------
    pandas.Dataframe, or csv file

    """

    for k, v in kwargs.items():
        if not isinstance(v, str):
            raise ValueError('{} must be a str. Instead got {}'
                .format(k, type(v)))

    method = '_get_{}_url'.format(cut_by)
    url = eval(method)(**kwargs)

    try:
        r = requests.get(url)
        if r.ok:
            pass
        else:
            raise requests.exceptions.HTTPError('Not a valid request.'
                'Double check for valid parameters.')
    except requests.exceptions.ConnectionError as e:
        print(e, '\nFailed to establish a new connection. Check internet.')

    if rtype == 'dataframe':
        return pd.read_csv(io.BytesIO(r.content), low_memory=False)
    elif rtype == 'csv':
        return io.BytesIO(r.content)
    else:
        raise ValueError('{} is not a valid return type\n' \
            'valid return types are `dataframe` or `io`' \
            .format(rtype))



import datetime

import pandas as pd

from . import util

# Previous year OES gets released in May
if datetime.date.today().month > 6:
    CUR_YEAR = datetime.date.today().year - 1
else:
    CUR_YEAR = datetime.date.today().year - 2

def _get_zip_urls(year):
    """Return urls for zip files based on pattern.

    Parameters
    ----------
    year : string
        4 digit year in string format 'YYYY'.

    Returns
    -------
    dict
    """

    suffix = f'oesm{year[2:]}'
    code = 4

    return {
        'national': f'https://www.bls.gov/oes/special.requests/{suffix}'
            'nat.zip',
        'state': f'https://www.bls.gov/oes/special.requests/{suffix}'
            'st.zip',
        'area': f'https://www.bls.gov/oes/special.requests/{suffix}ma.zip',
        'industry': f'https://www.bls.gov/oes/special.requests/{suffix}in'
            f'{code}.zip'}

def _get_filenames(year, industry_scope=None):
    """Return filenames for oes files based on pattern.

    Parameters
    ----------
    year : string
        4 digit year in string format 'YYYY'.

    Returns
    -------
    dict.

    """
    if industry_scope == None:
        prefix = 'natsector'
    elif (industry_scope == 5) or (industry_scope == 6):
        prefix = 'nat5d_6d'
    else:
        prefix = 'nat{}d'.format(industry_scope)

    return {
        'national': f'oesm{year[2:]}nat/national_M{year}_dl.xlsx',
        'state': f'oesm{year[2:]}st/state_M{year}_dl.xlsx',
        'metros': f'oesm{year[2:]}ma/MSA_M{year}_dl.xlsx',
        'metros-divisions': f'oesm{year[2:]}ma/aMSA_M{year}_dl.xlsx' ,
        'non-metros': f'oesm{year[2:]}ma/BOS_M{year}_dl.xlsx',
        'industry': f'oesm{year[2:]}in4/{prefix}_M{year}_dl.xlsx'
    }


def get_data(year=CUR_YEAR, cut_by='national', area_focus=None,
    industry_scope=None,rtype='dataframe', set_missing_figures_as_na=True):
    """Get a dataset from the Occupational Employment Statistics (OES).

    The OES database contains statistics for various occupations.
    Primary statistics include wage, salary and total employment by occupation.

    Default behavior returns occupational data at a national
    level. This behavior can be overridden by adding options.

    Intended usage is to return a pandas dataframe for easy data
    analysis. This can be overwridden to return a primitive i/o object for
    different data processing decisions.


    Parameters
    ----------
    year : str
        year must be in form 'YYYY'. This package only supports OES after 2014
    cut_by : str
        Get occupation figures that are either cut by location
        or industry. Only three options are allowed: `state`, `area`,
        or `industry`. If not specified, default behavior is
        to return nationwide figures.
    area_focus : str
        If `cut_by = area` then an `area_focus` must also be
        specifed. Options include: metros, metro-divisions, or non-metros
    industry_scope : int, or None
        If `cut_by = industry` then an `industry_scope` must also be specified.
        Options include 3, 4, 5 or 6 digit scope.
        If not specified, then the default is top level sectors (i.e 2 digit).
        Industry scope describes the level of industry granularity. Higher the
        scope, the more granular the industries will be.
    set_missing_figures_as_na : bool, Default = True
        BLS uses ['*', '**', '#', '~'] to designate missing non-numeric figures
        in columns that have numeric statistics, such as hourly wage. This
        mismatch converts the whole column to an object and not a number type.
        By keeping this argument to True, this symbols will get converted
        to NA values allowing these columns to be calculated and treated as
        numbers.
    rtype : str
        Default behavior is to return a pandas dataframe.
        If you want a primitive i/o object that supports open(),
        then specify rtype='io'.

    Returns
    -------
    pd.Dataframe, or io file like object

    Usage:

    >>> df = get_data(2017, cut_by='state')
    >>> df.head()
       AREA  ST    STATE OCC_CODE  ...   A_PCT75 A_PCT90 ANNUAL HOURLY
    0     1  AL  Alabama  00-0000  ...     52020   78690    NaN    NaN
    1     1  AL  Alabama  11-0000  ...    133360  188860    NaN    NaN
    2     1  AL  Alabama  11-1011  ...         #       #    NaN    NaN
    3     1  AL  Alabama  11-1021  ...    147860       #    NaN    NaN
    4     1  AL  Alabama  11-1031  ...     24630   47510   True    NaN

    [5 rows x 25 columns]

    """
    if int(year) < 2014:
        raise ValueError(f'Year:{year}. OES only supports years post 2014.')
    if int(year) > CUR_YEAR:
        raise ValueError(f'Year:{year}. OES data not available yet.')

    if industry_scope != None and industry_scope not in range(3, 7):
        raise ValueError(f'Industry scope:{industry_scope}. '
            'Not a valid industry scope. Options include ints from 3 to 6')

    OES_ZIP_URLS = _get_zip_urls(str(year))
    OES_FILENAMES = _get_filenames(str(year), industry_scope)

    zip_url = OES_ZIP_URLS.get(cut_by)
    if zip_url == None:
        raise ValueError('"{}" is not a valid cut_by option\n' \
            'valid options include:\n{}' \
            .format(cut_by, [k for k in OES_ZIP_URLS.keys()]))

    if cut_by == 'area':
        filename = OES_FILENAMES.get(area_focus)
        if filename == None:
            raise ValueError('"{}" is not a valid area focus\n' \
                'valid options include:\n{}' \
                .format(area_focus, ['metros', 'metros-divisions', 'non-metros']))
    else:
        filename = OES_FILENAMES.get(cut_by)

    fp = util.get_file(zip_url, filename)

    if set_missing_figures_as_na:
        na_values = ['*', '**', '#', '~']
    else:
        na_values = []
        
    if rtype == 'dataframe':
        return pd.read_excel(fp, na_values = na_values)
    elif rtype == 'io':
        return fp
    else:
        raise ValueError('{} is not a valid return type\n' \
            'valid return types are `dataframe` or `io`' \
            .format(rtype))



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='bls_datasets',
    version='0.0.9',
    description='Python library for retrieving BLS datasets',
    long_description_content_type='text/markdown',
    long_description=long_description,
    url='https://github.com/mbkupfer/bls-datasets',
    author='Maxim Kupfer',
    author_email='mbkupfer@gmail.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Financial and Insurance Industry',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Office/Business',
        'Topic :: Office/Business :: Financial',
        'Topic :: Scientific/Engineering',
    ],
    keywods='bls oes qcew',
    packages=['bls_datasets'],
    install_requires=[
        'pandas>=0.23.4',
        'xlrd>=1.1.0',
        'requests>=2.19.1',
    ],
)
